@@ -28,7 +28,7 @@
 extern "C" {
 #endif
 
-#define LIBTASN1_VERSION "0.2.9"
+#define LIBTASN1_VERSION "0.2.11"
 
 #include <sys/types.h>
 #include <time.h>
@@ -100,7 +100,7 @@ typedef int asn1_retCode;  /* type returned by libasn1 functions */
 
 /******************************************************/
 /* Structure definition used for the node of the tree */
-/* that rappresent an ASN.1 DEFINITION.               */
+/* that represent an ASN.1 DEFINITION.                */
 /******************************************************/
 typedef struct node_asn_struct{
   char *name;                    /* Node name */
@@ -147,25 +147,30 @@ asn1_retCode asn1_delete_structure(ASN1_TYPE *structure);
 
 asn1_retCode asn1_delete_element(ASN1_TYPE structure,const char *element_name);
 
-asn1_retCode asn1_write_value(ASN1_TYPE element,const char *name,
-			      const void *value,int len);
+asn1_retCode asn1_write_value(ASN1_TYPE *node_root,const char *name,
+			      const void *ivalue,int len);
 
-asn1_retCode asn1_read_value(ASN1_TYPE element,const char *name,void *value,
-                             int *len);
+asn1_retCode asn1_read_value(ASN1_TYPE *root,const char *name,
+			     void* ivalue,int *len);
 
 asn1_retCode asn1_number_of_elements(ASN1_TYPE element,const char *name,int *num);
 
 asn1_retCode asn1_der_coding(ASN1_TYPE element,const char *name,
-               void *der,int *len,char *ErrorDescription);
+			     void *ider,int *len,
+			     char *ErrorDescription);
 
-asn1_retCode asn1_der_decoding(ASN1_TYPE *element,const void *der,int len,
-                 char *errorDescription);
+asn1_retCode asn1_der_decoding(ASN1_TYPE *element,const void *ider,int len,
+			       char *errorDescription);
 
-asn1_retCode asn1_der_decoding_element(ASN1_TYPE *structure,const char *elementName,
-		 const void *der,int len,char *errorDescription);
+asn1_retCode asn1_der_decoding_element(ASN1_TYPE *structure,
+				       const char *elementName,
+				       const void *ider,int len,
+				       char *errorDescription);
 
-asn1_retCode asn1_der_decoding_startEnd(ASN1_TYPE element,const void *der,
-               int len,const char *name,int *start, int *end);
+asn1_retCode asn1_der_decoding_startEnd(ASN1_TYPE element,
+					const void *ider,int len,
+					const char *name_element,
+					int *start,int *end);
 
 asn1_retCode asn1_expand_any_defined_by(ASN1_TYPE definitions,
                ASN1_TYPE *element);
@@ -190,16 +195,3 @@ void libtasn1_perror(asn1_retCode error);
 #endif
 
 #endif /* LIBASN1_H */
-
-
-
-
-
-
-
-
-
-
-
-
-
