@@ -91,7 +91,7 @@ _asn1_add_node(unsigned int type)
 /* Parameters:                                                    */
 /*   pointer: NODE_ASN element pointer.                           */
 /*   name: null terminated string with the element's name to find.*/
-/* Return: the searching result. NULL if not find.                */
+/* Return: the searching result. NULL if not found.               */
 /******************************************************************/
 node_asn *
 _asn1_find_node(node_asn *pointer,const char *name)
@@ -107,7 +107,7 @@ _asn1_find_node(node_asn *pointer,const char *name)
   p=pointer;
   n_start=name;
 
-  if(p->name != NULL){ /* has *pointer a name ? */
+  if(p->name != NULL){ /* has *pointer got a name ? */
     n_end=strchr(n_start,'.');     /* search the first dot */
     if(n_end){
       memcpy(n,n_start,n_end-n_start);
@@ -171,7 +171,7 @@ _asn1_find_node(node_asn *pointer,const char *name)
 /******************************************************************/
 /* Function : _asn1_set_value                                     */
 /* Description: sets the field VALUE in a NODE_ASN element. The   */
-/*              previus value (if exist) will be lost             */
+/*              previous value (if exist) will be lost            */
 /* Parameters:                                                    */
 /*   node: element pointer.                                       */
 /*   value: pointer to the value that you want to set.            */
@@ -198,7 +198,7 @@ _asn1_set_value(node_asn *node,const unsigned char *value,unsigned int len)
 /******************************************************************/
 /* Function : _asn1_set_name                                      */
 /* Description: sets the field NAME in a NODE_ASN element. The    */
-/*              previus value (if exist) will be lost             */
+/*              previous value (if exist) will be lost            */
 /* Parameters:                                                    */
 /*   node: element pointer.                                       */
 /*   name: a null terminated string with the name that you want   */
@@ -518,8 +518,8 @@ asn1_retCode
 _asn1_expand_object_id(ASN1_TYPE node)
 {
   node_asn *p,*p2,*p3,*p4,*p5;
-  char name_root[129],name2[129];
-  int move;
+  char name_root[MAX_NAME_SIZE],name2[2*MAX_NAME_SIZE+1];
+  int move, tlen;
  
   if(node==NULL) return ASN1_ELEMENT_NOT_FOUND;
 
@@ -548,7 +548,9 @@ _asn1_expand_object_id(ASN1_TYPE node)
 	      if(type_field(p4->type)==TYPE_CONSTANT){
 		p5=_asn1_add_node_only(TYPE_CONSTANT);
 		_asn1_set_name(p5,p4->name);
-		_asn1_set_value(p5,p4->value,strlen(p4->value)+1);
+		tlen = strlen( p4->value);
+		if (tlen > 0)
+		    _asn1_set_value(p5,p4->value,tlen+1);
 		if(p2==p){
 		  _asn1_set_right(p5,p->down);
 		  _asn1_set_down(p,p5);
@@ -584,6 +586,57 @@ _asn1_expand_object_id(ASN1_TYPE node)
     if(move==UP) p=_asn1_find_up(p);
   }
 
+
+  /*******************************/
+  /*       expand DEFAULT        */
+  /*******************************/
+  p=node;
+  move=DOWN;
+
+  while(!((p==node) && (move==UP))){
+    if(move!=UP){
+      if((type_field(p->type)==TYPE_OBJECT_ID) && 
+	 (p->type&CONST_DEFAULT)){
+	p2=p->down;
+        if(p2 && (type_field(p2->type)==TYPE_DEFAULT)){
+	  _asn1_str_cpy(name2, sizeof(name2), name_root);
+	  _asn1_str_cat(name2, sizeof(name2), ".");
+	  _asn1_str_cat(name2, sizeof(name2), p2->value);
+	  p3=_asn1_find_node(node,name2);
+	  if(!p3 || (type_field(p3->type)!=TYPE_OBJECT_ID) ||
+	     !(p3->type&CONST_ASSIGN)) return ASN1_ELEMENT_NOT_FOUND;
+	  p4=p3->down;
+	  name2[0]=0;
+	  while(p4){
+	    if(type_field(p4->type)==TYPE_CONSTANT){
+	      if(name2[0]) _asn1_str_cat(name2,sizeof(name2),".");
+	      _asn1_str_cat(name2,sizeof(name2),p4->value);
+	    }
+	    p4=p4->right;
+	  }
+	  tlen = strlen(name2);
+	  if (tlen > 0)
+	      _asn1_set_value(p2,name2,tlen+1);
+	}
+      }
+      move=DOWN;
+    }
+    else move=RIGHT;
+
+    if(move==DOWN){
+      if(p->down) p=p->down;
+      else move=RIGHT;
+    }
+    
+    if(p==node) {move=UP; continue;}
+
+    if(move==RIGHT){
+      if(p->right) p=p->right;
+      else move=UP;
+    }
+    if(move==UP) p=_asn1_find_up(p);
+  }
+
   return ASN1_SUCCESS;
 }
 
@@ -674,6 +727,22 @@ _asn1_check_identifier(ASN1_TYPE node)
 	return ASN1_IDENTIFIER_NOT_FOUND;
       } 
     }
+    else if((type_field(p->type)==TYPE_OBJECT_ID) && 
+	    (p->type&CONST_DEFAULT)){
+      p2=p->down;
+      if(p2 && (type_field(p2->type)==TYPE_DEFAULT)){
+	_asn1_str_cpy(name2, sizeof(name2), node->name);
+	_asn1_str_cat(name2, sizeof(name2), ".");
+	_asn1_str_cat(name2, sizeof(name2), p2->value);
+	strcpy(_asn1_identifierMissing,p2->value);
+	p2=_asn1_find_node(node,name2);
+	if(!p2 || (type_field(p2->type)!=TYPE_OBJECT_ID) ||
+	   !(p2->type&CONST_ASSIGN))
+	  return ASN1_IDENTIFIER_NOT_FOUND;
+	else
+	  _asn1_identifierMissing[0]=0;
+      }
+    }
     else if((type_field(p->type)==TYPE_OBJECT_ID) && 
 	    (p->type&CONST_ASSIGN)){
       p2=p->down;
@@ -723,9 +792,9 @@ _asn1_check_identifier(ASN1_TYPE node)
 /* Parameters:                                                    */
 /*   node: pointer to a DEFINITIONS element.                      */
 /* Return:                                                        */
-/*   ASN1_ELEMENT_NOT_FOUND if NODE is NULL or not a pointer to    */
+/*   ASN1_ELEMENT_NOT_FOUND if NODE is NULL or not a pointer to   */
 /*     a DEFINITIONS element,                                     */
-/*   otherwise ASN1_SUCCESS                                             */
+/*   otherwise ASN1_SUCCESS                                       */
 /******************************************************************/
 asn1_retCode 
 _asn1_set_default_tag(ASN1_TYPE node)
@@ -768,4 +837,79 @@ _asn1_set_default_tag(ASN1_TYPE node)
 
 
 
+static const char*
+parse_version_number( const char *s, int *number )
+{
+    int val = 0;
 
+    if( *s == '0' && isdigit(s[1]) )
+	return NULL; /* leading zeros are not allowed */
+    for ( ; isdigit(*s); s++ ) {
+	val *= 10;
+	val += *s - '0';
+    }
+    *number = val;
+    return val < 0? NULL : s;
+}
+
+/* The parse version functions were copied from libgcrypt.
+ */
+static const char *
+parse_version_string( const char *s, int *major, int *minor, int *micro )
+{
+    s = parse_version_number( s, major );
+    if( !s || *s != '.' )
+	return NULL;
+    s++;
+    s = parse_version_number( s, minor );
+    if( !s || *s != '.' )
+	return NULL;
+    s++;
+    s = parse_version_number( s, micro );
+    if( !s )
+	return NULL;
+    return s; /* patchlevel */
+}
+
+/**
+  * asn1_check_version - This function checks the library's version
+  * @req_version: the version to check
+  *
+  * Check library version.
+  *
+  * Return value: Check that the the version of the library is at
+  *   minimum the one given as a string in @req_version and return the
+  *   actual version string of the library; return NULL if the
+  *   condition is not met.  If %NULL is passed to this function no
+  *   check is done and only the version string is returned.
+  **/
+const char *
+asn1_check_version( const char *req_version )
+{
+    const char *ver = LIBTASN1_VERSION;
+    int my_major, my_minor, my_micro;
+    int rq_major, rq_minor, rq_micro;
+    const char *my_plvl, *rq_plvl;
+
+    if ( !req_version )
+	return ver;
+
+    my_plvl = parse_version_string( ver, &my_major, &my_minor, &my_micro );
+    if ( !my_plvl )
+	return NULL;  /* very strange our own version is bogus */
+    rq_plvl = parse_version_string( req_version, &rq_major, &rq_minor,
+								&rq_micro );
+    if ( !rq_plvl )
+	return NULL;  /* req version string is invalid */
+
+    if ( my_major > rq_major
+	|| (my_major == rq_major && my_minor > rq_minor)
+	|| (my_major == rq_major && my_minor == rq_minor
+				 && my_micro > rq_micro)
+	|| (my_major == rq_major && my_minor == rq_minor
+				 && my_micro == rq_micro
+				 && strcmp( my_plvl, rq_plvl ) >= 0) ) {
+	return ver;
+    }
+    return NULL;
+}
